/**************************************************************
 *
 *  Program Title:  WaspARM Monitor
 *
 *  Program Name:  Accelerometer.c	  Version: 0.80
 *
 *  Author:  BR	  					  Date:  March 26, 2008
 *
 *
 *  This module handles all accelerometer setup and processing.
 *
 * Copyright 2009, SOC ROBOTICS, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC ROBOTICS,
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS,
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */
#include "board.h"
#include "Wasp_Arm_Utilities.h"
#include "Accelerometer.h"

/**
 * Macros
 */
#define SET_GS1() pPIO->PIO_SODR = GS1
#define SET_GS2() pPIO->PIO_SODR = GS2
#define SET_SLEEP() pPIO->PIO_SODR = SLEEP
#define CLEAR_GS1() pPIO->PIO_CODR = GS1
#define CLEAR_GS2() pPIO->PIO_CODR = GS2
#define CLEAR_SLEEP()  pPIO->PIO_CODR = SLEEP

#define READ_X_VALUE() ad[X_AXIS_PORT]
#define READ_Y_VALUE() ad[Y_AXIS_PORT]
#define READ_Z_VALUE() ad[Z_AXIS_PORT]

static volatile AT91PS_PIO pPIO = AT91C_BASE_PIOA;// pointer to PIO data structure

//This is the factor that maps our voltage reading to G reading for the accelerometer
static float sensitivity;
//We assume that there is no offset in our accelerometer and 0G = half the power rail
static float x_offset = 0.5;
static float y_offset = 0.5;
static float z_offset = 0.5;
//We shall store our current g reading here
float gvalues[3];

/**
 * This method will set our port pins as outputs
 */
void setAccelerometerPorts(void)
{
	pPIO->PIO_PER = SLEEP | GS1 | GS2;
	pPIO->PIO_OER = SLEEP | GS1 | GS2;
}

/*
 * This method will allow you to change the accelerometers sensitivity
 * ACCELEROMETER_1g5 -> 1.5g range
 * ACCELEROMETER_2g -> 3g range
 * ACCELEROMETER_4g -> 4g range
 * ACCELEROMETER_6g -> 6g range
 */

void setAccelerometerRange(unsigned char range)
{
	if (range == ACCELEROMETER_1g5)
	{
		  CLEAR_GS1();//clear GS1 and GS2
		  CLEAR_GS2();
		  sensitivity = ACCELEROMETER_1g5_SENSITIVITY;
	}
	else if (range == ACCELEROMETER_2g)
	{
		  CLEAR_GS2();//clear GS2
		  SET_GS1();//set GS1
		  sensitivity = ACCELEROMETER_2g_SENSITIVITY;
	}
	else if (range == ACCELEROMETER_4g)
	{
		  CLEAR_GS1();//Clear GS1
		  SET_GS2();//Set GS2
		  sensitivity = ACCELEROMETER_4g_SENSITIVITY;
	}
	else if (range == ACCELEROMETER_6g)
	{
	 	  SET_GS1();//Set GS1 & GS2
		  SET_GS2();
		  sensitivity = ACCELEROMETER_6g_SENSITIVITY;
	}
}


/**
 * This method will either enable or disable the accelerometer
 */

void enableAccelerometer(unsigned char sleep)
{
 	 if(sleep == OFF){
	 		  CLEAR_SLEEP(); // Disable Accelerometer
	 }
	 else
	 {
	  	 	  SET_SLEEP(); // Enable Accelerometer
	 }

}

/**
 * Call this method to estimate the offset that the accelerometer has.
 * The board should be level and still before you call this function
 * It works by assuming that X and Y should = 0 G and Z should equal 1 G
 */
void zeroOffset(void)
{
	int i;
	long countx = 0, county = 0, countz = 0;

	//find an average stable reading
	for (i=0;i<REFERENCE_SAMPLE;i++)
	{
		Read_ADC();
		countx += READ_X_VALUE();
		county += READ_Y_VALUE();
		countz += READ_Z_VALUE();
	}

	//find out the fraction it is of the power rail (i.e. divide by the resolution of the ADC
	x_offset = countx/(REFERENCE_SAMPLE*1024.0);
	y_offset = county/(REFERENCE_SAMPLE*1024.0);

	//if device is on top of board use this statement
	//we should have 1g force on the z axis
	z_offset = ((countz/(float)REFERENCE_SAMPLE)-(sensitivity*1024.0))/1024.0;

	//if device is on the bottom of board use this statement
	//we should have -1g force on the z axis
	//z_offset = ((countz/(float)REFERENCE_SAMPLE)+(sensitivity*1024.0))/1024.0;
}

/**
 * This method will sample all the accelerometers outputs and will calculate and store
 * the corresponding G readings
 */
void readAccelerometer(void)
{


	int i;
	int countx = 0, county = 0, countz = 0;

	//find an average stable reading
	for (i=0;i<SAMPLE_SIZE;i++)
	{
		Read_ADC();
		countx += READ_X_VALUE();
		county += READ_Y_VALUE();
		countz += READ_Z_VALUE();
	}

	/*
	 * Here is how we work out the acceleration
     *        (ADC/1024) - 0.5
     * g =   -----------------
	 *               s
     *
	 * Please note that this funtion does not calibrate.
     * It assumes that 0g is at the mid point of the voltage rails.
	 * However 0g on a 3.3v supply can vary from 1.485 to 1.815 typ 1.65
	 *
	 *You should call zeroOffset() to calibrate it to 0g
	 */

	gvalues[X_AXIS] = ((countx/(SAMPLE_SIZE*1024.0))-x_offset)/sensitivity;
	gvalues[Y_AXIS] = ((county/(SAMPLE_SIZE*1024.0))-y_offset)/sensitivity;
	gvalues[Z_AXIS] = ((countz/(SAMPLE_SIZE*1024.0)-z_offset))/sensitivity;
}

/**
 * This function will return a float corresponding the G reading for the axis selected
 */
float getReading(unsigned char axis)
{
	return gvalues[axis];
}

/**
 * This function will print the selected axis value.
 */
void printvalue(unsigned char axis, unsigned char *text)
{
	int i = 0, hundreds, tens, intvalue, decvalue;

	float g_reading = gvalues[axis];

	if(g_reading < 0.0)
	{
		text[i++] = '-';
		g_reading *= -1.0;
	}

	intvalue = (int)g_reading;

	g_reading -= (float) intvalue;

	g_reading *= 1000.0;//I will want 2 decimal places

	decvalue = (int)g_reading;

	Print_Int(intvalue, &text[i], DEC);


	//find the end of the int value
	while (text[i] != 0x00)
	{
		i++;
	}

	text[i++] = '.';

	hundreds = decvalue/100;
	decvalue -= (hundreds*100);
	tens = decvalue/10;

	text[i++] = '0' + hundreds;
	text[i++] = '0' + tens;
	text[i++] = '0' + decvalue%10;
	text[i] = 0x00;

}


